<?php echo "<?php\n"; ?>
/********************************************************************/
/* @copyright Copyright (c) 2012-<?php echo date('Y')?>, YOUR COPYRIGHT NAME */
/********************************************************************/
<?php
/**
 * This is the template for generating the model class of a specified table.
 * In addition to the default model Code, this adds the CSaveRelationsBehavior
 * to the model class definition.
 * - $this: the ModelCode object
 * - $tableName: the table name for this class (prefix is already removed if necessary)
 * - $modelClass: the model class name
 * - $columns: list of table columns (name=>CDbColumnSchema)
 * - $labels: list of attribute labels (name=>label)
 * - $rules: list of validation rules
 * - $relations: list of relations (name=>relation declaration)
 */
?>
/**
 * This is the model base class for the table "<?php echo $tableName; ?>".
 *
 * Columns in table "<?php echo $tableName; ?>" available as properties of the model:
<?php
    foreach($columns as $key => $column) {
    	print " *  @property $column->type \$$column->name\n";
}
?>
<?php if(count($relations)>0): ?>
 *
 * Relations of table "<?php echo $tableName; ?>" available as properties of the model:
<?php else: ?>
 * There are no model relations.
<?php endif; ?>
<?php foreach($relations as $name=>$relation): ?>
 *  @property <?php
    if (preg_match("~^array\(self::([^,]+), '([^']+)', '([^']+)'\)$~", $relation, $matches))
    {
        $relationType = $matches[1];
        $relationModel = $matches[2];

        switch($relationType){
            case 'HAS_ONE':
                echo $relationModel.' $'.$name."\n";
            break;
            case 'BELONGS_TO':
                echo $relationModel.' $'.$name."\n";
            break;
            case 'HAS_MANY':
                echo $relationModel.'[] $'.$name."\n";
            break;
            case 'MANY_MANY':
                echo $relationModel.'[] $'.$name."\n";
            break;
            default:
                echo 'mixed $'.$name."\n";
        }
    }
    ?>
<?php endforeach; ?>
 *
 * Columns in table "<?php echo $tableName; ?>" are available as scopes of the model (with RelatedSearchBehavior):
<?php
    foreach($columns as $key => $column) {
    	print " *  @method $modelClass {$column->name}() {$column->name}(mixed \$value, boolean \$partialMatch=false, string \$operator='AND', boolean \$escape=true)\n";
}
?>
 * Documentation for default methods to provide the class specific method signature.
 *  @method <?php echo $modelClass?> resetScope() resetScope(boolean $resetDefault=true)
 */
abstract class <?php echo 'Base' . $modelClass; ?> extends <?php echo $this->baseClass; ?> {

    /* Constants to reference table columns */
<?php
foreach($columns as $name=>$column) {
	echo "    const ".strtoupper($column->name)." = '{$column->name}'; // {$labels[$name]}\n";
}
?>

    public static function model($className = __CLASS__) {
        return parent::model($className);
    }

    public function tableName() {
        return '<?php echo $tableName; ?>';
    }

    public function rules() {
        return array(
<?php
        foreach($rules as $rule) {
            echo "            $rule,\n";
        }
?>
            array('<?php echo implode(', ', array_keys($columns)); ?>', 'safe', 'on' => 'search'),
        );
    }

    public function __toString() {
        return (string) $this-><?php
            $found = false;
        foreach($columns as $name => $column) {
            if(!$found
                    && $column->type != 'datetime'
                    && $column->type==='string'
                    && !$column->isPrimaryKey){
                echo $column->name;
                $found = true;
            }
        }
        // if the columns contains no column of type 'string', return the
        // first column (usually the primary key)
        if(!$found)
            echo reset($columns)->name;
        ?>;
    }

    /**
     * Updates records with the specified condition.
     *
     * Modifies the standard method in that it takes into account the model conditions to select the records to update.
     *
     * @see CActiveRecord::updateAll()
     */
	public function updateAll($attributes,$condition=array(),$params=array())
	{
        //Next line must be added before defining the criteria/scopes.
        //<?php echo $modelClass;?>::model()->getDbCriteria()->alias=<?php echo $modelClass;?>::model()->tableName();
		$crit=new CDbCriteria($condition);
		$this->applyScopes($crit);
		return parent::updateAll($attributes,$crit);
    }

    /**
     * Deletes records with the specified condition.
     *
     * Modifies the standard method in that it takes into account the model conditions to select the records to update.
     *
     * @see CActiveRecord::updateAll()
     */
    public function deleteAll($condition=array(),$params=array())
    {
        //Next line must be added before defining the criteria/scopes.
        //<?php echo $modelClass;?>::model()->getDbCriteria()->alias=<?php echo $modelClass;?>::model()->tableName();
        $crit=new CDbCriteria($condition);
        $this->applyScopes($crit);
        return parent::deleteAll($crit);
    }

  	/**
	 * Unset attributes of model when this is a search scenario
	 * to avoid unwanted side effects of initial values set
	 * in the database.
	 *
     * @see CActiveRecord::afterConstruct()
     */
	protected function afterConstruct() {
	    if($this->getScenario()==='search') {
	        $this->unsetAttributes();
	    }
        parent::afterConstruct();
	}

    public function behaviors() {
        <?php
            $behaviors = "return array(\n";
                    foreach($columns as $name => $column) {
                    if(in_array($column->name, array(
                                'create_time',
                                'createtime',
                                'created_at',
                                'createdat',
                                'changed',
                                'changed_at',
                                'updatetime',
                                'update_time',
                                'timestamp'))) {
                    $behaviors .= sprintf("\n                    'CTimestampBehavior' => array(
                        'class' => 'zii.behaviors.CTimestampBehavior',
                        'createAttribute' => %s,
                        'updateAttribute' => %s,
                    ),\n", $this->getCreatetimeAttribute($columns),
                        $this->getUpdatetimeAttribute($columns));
                    break; // once a column is found, we are done
                    }
                    }

//                    foreach($columns as $name => $column) {
//                        if(in_array($column->name, array(
//                                        'user_id',
//                                        'userid',
//                                        'ownerid',
//                                        'owner_id',
//                                        'created_by',
//                                        'createdby'))) {
//                            $behaviors .= sprintf("\n        'OwnerBehavior' => array(
//                                'class' => 'OwnerBehavior',
//                            'ownerColumn' => '%s',
//                                ),\n", $column->name);
//                            break; // once a column is found, we are done
//
//                        }
//                    }

                    if (count($relations)){
                        //$behaviors .= "\n        'activerecord-relation' => array('class' => 'EActiveRecordRelationBehavior')"."\n";
                    }

                    $behaviors.=<<<EOBH
            'ERememberFiltersBehavior' => array(
                'class' => 'ext.ERememberFiltersBehavior.ERememberFiltersBehavior',
                'defaults'=>array(),
                'defaultStickOnClear'=>false,
            ),
            'relatedsearch'=>array(
                    'class'=>'RelatedSearchBehavior',
                    'relations'=>array(
                            /* Examples:
                            'owner'=>'ownerUser.displayname',
                            'serial'=>array('field'=>'device.device_identifier','searchvalue'=>'serialSearch'),
                            */
                            /* Fields for easy access */
                    ),
            ),

EOBH;

                    $behaviors .= ");\n";
                    echo $behaviors;
                    ?>
    }

    public function relations() {
        return array(
<?php
        foreach($relations as $name=>$relation) {
            echo "            '$name' => $relation,\n";
        }
?>
        );
    }

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
<?php foreach($labels as $name=>$label): ?>
			<?php echo "'$name' => Yii::t('app','lb.$tableName.$name'), // $label\n"; ?>
<?php endforeach; ?>
		);
	}

    /*
     * @return KeenActiveDataProvider
     */
    public function search() {
        $criteria = new CDbCriteria;
        $t=$this->getTableAlias(false,false);
        $ds=$this->getDbConnection()->getSchema();

<?php
        foreach($columns as $name=>$column)
        {
            if($column->type==='string' and !$column->isForeignKey)
            {
                echo "        \$criteria->compare(\$ds->quoteColumnName(\"\$t.$name\"), \$this->$name, true);\n";
            }
            else
            {
                echo "        \$criteria->compare(\$ds->quoteColumnName(\"\$t.$name\"), \$this->$name);\n";
            }
        }
        echo "\n";
?>
		return $this->relatedSearch(
		    $criteria
		    //,array('sort'=>array('defaultOrder'=>$this->getTableAlias(true,false).'.entity_id DESC'))
		);
        /*
        return new CActiveDataProvider(get_class($this), array(
                    'criteria' => $criteria,
                ));
        */
    }

}